############################################################################
# Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2011. #
# All Rights Reserved.                                                     #
#                                                                          #
# Pre-requisite: WAS and CLM should be installed                           #
# This script installs all CLM apps to a single WAS server                 #
#                                                                          #
# Usage (Win):                                                             #
#  wsadmin -language jython -user <username> -password <password>          #
#    -f <path to clm_deploy.py e.g.                                        #
#        C:/JazzTeamServer/server/was/clm_deploy.py>                       #
#    <nodename>                                                            #
#    <servername>                                                          #
#    <path to war files e.g. C:/JazzTeamServer/server/webapps>             #
#                                                                          #
# Usage (Linux):                                                           #
#  ./wsadmin.sh -language jython -user <username> -password <password>     #
#    -f <path to clm_deploy.py e.g.                                        #
#        /opt/IBM/JazzTeamServer/server/was/clm_deploy.py>                 #
#    <nodename>                                                            #
#    <servername>                                                          #
#    <path to war files e.g. /opt/IBM/JazzTeamServer/server/webapps>       #
############################################################################

import os
import re

# Keep track of any errors/exceptions raised during the app installation.
returncode = 0

# retrieve all .war files from directory
def constructAppsList(dir, fileExtension):
	global returncode
	apps = []
	if os.path.exists(dir):
		for filename in os.listdir(dir):
			if filename.endswith(fileExtension):
				apps.append(filename.replace(fileExtension, ""))
			#endIf
		#endFor
	else:
		print "Error: the following directory is invalid: "+dir
		# indicate an error due to invalid directory
		returncode = 2
	return apps
#endDef

# Determine the offsets of the arguments
if (sys.argv[0] == '-language'):
	firstArgIndex = 2
else:
	firstArgIndex = 0

# Get the node from commandline
node = sys.argv[firstArgIndex+0]

# Find the cell name
cell = AdminControl.getCell()

# Get the server name from commandline
server = sys.argv[firstArgIndex+1]
# Get the location of the war files
war_location = sys.argv[firstArgIndex+2]

# Location of class path for additional configuration (rs only)
m = re.search(r'(.*)\/(server)', war_location)
path_prefix = m.group(1)+"/"+m.group(2)

class_path = os.path.join(path_prefix,"conf", "rs", "WAS_SharedLibrary")

configuration_path = path_prefix+"/conf/"

# Under Liberty, the WAR files follow the *.war.zip file naming pattern
# The script will check whether Liberty is being used, and use the correct file naming pattern accordingly (.war or .war.zip).
warFileExtension=".war"

if war_location.find("clmServerTemplate/apps") != -1 or war_location.find("clmServerTemplate\\apps") != -1:
	warFileExtension=".war.zip"

#######################
# deploy all the apps #
#######################

# Get available apps from war_location
apps = constructAppsList(war_location, warFileExtension)

for app in apps:
	try:
		print "Installing "+app

		if app != 'lqe' and app != 'ldx':
			full_path_to_app = war_location + "/" +app+warFileExtension
			attrs = ['-appname', app, '-defaultbinding.virtual.host default_host -usedefaultbindings', '-node', node, '-server', server, '-contextroot',  "/"+app]
			error = AdminApp.install(full_path_to_app , attrs)
			AdminConfig.save()

		# Check to see if the CLM apps to be deployed contains any of the following:
		# JRS, LQE, LDX
		# These apps require additional configuration in order to be fully deployed
		# The additional configuration will be conducted below

		# Configuration for RS

		if app == "rs":
			AdminConfig.create('Library', AdminConfig.getid('/Cell:'+cell+'/Node:'+node+'/Server:'+server+'/'), '[[nativePath ""] [name "JRS Shared Library"] [isolatedClassLoader true] [description ""] [classPath "'+class_path+'"]]')
			AdminApp.edit('rs', '[ -MapSharedLibForMod [[ rs META-INF/application.xml "JRS Shared Library" ]]]' )
			dep = AdminConfig.getid('/Deployment:rs/')
			depObject = AdminConfig.showAttribute(dep, 'deployedObject')
			classldr = AdminConfig.showAttribute(depObject, 'classloader')
			AdminConfig.modify(classldr, [['mode', 'PARENT_LAST']])
			AdminConfig.save()

		# Configuration for LQE

		if app == 'lqe':
			AdminConfig.create('Property', '(cells/'+cell+'/nodes/'+node+'/servers/'+server+'|server.xml#JavaVirtualMachine_1183122130078)', '[[validationExpression ""] [name "lqe.config.location"] [description ""] [value "file:///'+configuration_path+'lqe"] [required "false"]]')
			AdminApp.install(war_location+'/lqe'+warFileExtension, '[ -usedefaultbindings -defaultbinding.virtual.host default_host -nopreCompileJSPs -distributeApp -nouseMetaDataFromBinary -nodeployejb -appname lqe -createMBeansForResources -noreloadEnabled -nodeployws -validateinstall warn -noprocessEmbeddedConfig -filepermission .*\.dll=755#.*\.so=755#.*\.a=755#.*\.sl=755 -noallowDispatchRemoteInclude -noallowServiceRemoteInclude -asyncRequestDispatchType DISABLED -nouseAutoLink -noenableClientModule -clientMode isolated -novalidateSchema -contextroot /lqe -MapModulesToServers [[ lqe lqe'+warFileExtension+',WEB-INF/web.xml WebSphere:cell='+cell+',node='+node+',server='+server+' ]] -CtxRootForWebMod [[ lqe lqe'+warFileExtension+',WEB-INF/web.xml /lqe ]]]' )
			AdminApp.edit('lqe', '[ -MapModulesToServers [[ lqe lqe'+warFileExtension+',WEB-INF/web.xml WebSphere:cell='+cell+',node='+node+',server='+server+' ]]]' )
			dep = AdminConfig.getid('/Deployment:lqe/')
			depObject = AdminConfig.showAttribute(dep, 'deployedObject')
			classldr = AdminConfig.showAttribute(depObject, 'classloader')
			policy = AdminConfig.showAttribute(depObject, 'warClassLoaderPolicy')
			AdminConfig.modify(classldr, [['mode', 'PARENT_LAST']])
			AdminConfig.modify(depObject, [['warClassLoaderPolicy', 'SINGLE']])
			lqeModule = AdminConfig.showAttribute(depObject, 'modules')
			lqeModule = lqeModule[1:len(lqeModule)-1]
			AdminConfig.modify(lqeModule, [['classloaderMode', 'PARENT_LAST']])
			AdminConfig.save()

		# Configuration for LDX

		if app == 'ldx':
			AdminConfig.create('Property', '(cells/'+cell+'/nodes/'+node+'/servers/'+server+'|server.xml#JavaVirtualMachine_1183122130078)', '[[validationExpression ""] [name "ldx.config.location"] [description ""] [value "file:///'+configuration_path+'ldx"] [required "false"]]')
			AdminApp.install(war_location+'/ldx'+warFileExtension, '[ -usedefaultbindings -defaultbinding.virtual.host default_host -nopreCompileJSPs -distributeApp -nouseMetaDataFromBinary -nodeployejb -appname ldx -createMBeansForResources -noreloadEnabled -nodeployws -validateinstall warn -noprocessEmbeddedConfig -filepermission .*\.dll=755#.*\.so=755#.*\.a=755#.*\.sl=755 -noallowDispatchRemoteInclude -noallowServiceRemoteInclude -asyncRequestDispatchType DISABLED -nouseAutoLink -noenableClientModule -clientMode isolated -novalidateSchema -contextroot /ldx -MapModulesToServers [[ ldx ldx'+warFileExtension+',WEB-INF/web.xml WebSphere:cell='+cell+',node='+node+',server='+server+' ]] -CtxRootForWebMod [[ ldx ldx'+warFileExtension+',WEB-INF/web.xml /ldx ]]]' )
			AdminApp.edit('ldx', '[ -MapModulesToServers [[ ldx ldx'+warFileExtension+',WEB-INF/web.xml WebSphere:cell='+cell+',node='+node+',server='+server+' ]]]' )
			dep = AdminConfig.getid('/Deployment:ldx/')
			depObject = AdminConfig.showAttribute(dep, 'deployedObject')
			classldr = AdminConfig.showAttribute(depObject, 'classloader')
			policy = AdminConfig.showAttribute(depObject, 'warClassLoaderPolicy')
			AdminConfig.modify(classldr, [['mode', 'PARENT_LAST']])
			AdminConfig.modify(depObject, [['warClassLoaderPolicy', 'SINGLE']])
			ldxModule = AdminConfig.showAttribute(depObject, 'modules')
			ldxModule = ldxModule[1:len(ldxModule)-1]
			AdminConfig.modify(ldxModule, [['classloaderMode', 'PARENT_LAST']])
			AdminConfig.save()

	except:
		_type_, _value_, _tbck_ = sys.exc_info()
		error = `_value_`
		print "Error Installing Application "
		print "Error Message = "+error
		#indicate an error due to an unsuccessful installation
		returncode = 1
	#endTry
#endFor
print "The installed applications may be started from the Integrated Solutions Console in WebSphere."

# Example of how to set role/mapping for jts.war
# AdminApp.edit('jts.war', '[-MapRolesToUsers [["JazzAdmins" No No "" admins]["JazzDWAdmins" No No "" admins]["JazzUsers" No No "" users]["JazzGuests" No No "" users]["JazzProjectAdmins" No No "" admins]]]')
# AdminConfig.save()

# Notify the caller whether all applications were installed successfully
sys.exit(returncode)