############################################################################
# Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2011. #
# All Rights Reserved.                                                     #
#                                                                          #
# Pre-requisite: WAS and CLM should be installed                           #
# This script installs the list of CLM apps to a single WAS server         #
#                                                                          #
# Usage (Win):                                                             #
#  wsadmin -language jython -user <username> -password <password>          #
#    -f <path to clm_deploy_distributed.py e.g.                            #
#        C:/JazzTeamServer/server/was/clm_deploy_distributed.py>           #
#    <nodename>                                                            #
#    <servername>                                                          #
#    <path to war files e.g. C:/JazzTeamServer/server/webapps>             #
#    <comma-separated list (no spaces) of apps to be deployed e.g. jts,ccm>#
#																		   #
# Usage (Linux):                                                           #
#  ./wsadmin.sh -language jython -user <username> -password <password>     #
#    -f <path to clm_deploy_distributed.py e.g.                            #
#        /opt/IBM/JazzTeamServer/server/was/clm_deploy_distributed.py>     #
#    <nodename>                                                            #
#    <servername>                                                          #
#    <path to war files e.g. /opt/IBM/JazzTeamServer/server/webapps>       #
#    <comma-separated list (no spaces) of apps to be deployed e.g. jts,ccm>#
#    *RS should be installed before LDX/LQE. Please specify RS in the list #
#    * of apps to be deployed BEFORE LDX or LQE                            #
############################################################################

import os.path
import re

# Keep track of any errors/exceptions raised during the app installation.
returncode = 0

# Determine the offsets of the arguments
if (sys.argv[0] == '-language'):
	firstArgIndex = 2
else:
	firstArgIndex = 0

# Get the node from commandline
node = sys.argv[firstArgIndex]
# Find the cell name
cell = AdminControl.getCell()

# Get the server name from commandline
server = sys.argv[firstArgIndex+1]
# Get the location of the war files
war_location = sys.argv[firstArgIndex+2]

# list of CLM apps to be deployed
apps=sys.argv[firstArgIndex+3]
apps=apps [0:len(apps)].split(",")

# location of class path for additonal configuration (rs only)
# Location of class path for additional configuration (rs only)
m = re.search(r'(.*)\/(server)', war_location)
path_prefix = m.group(1)+"/"+m.group(2)

class_path = os.path.join(path_prefix,"conf", "rs", "WAS_SharedLibrary")

configuration_path = path_prefix+"/conf/"

# Under Liberty, the WAR files follow the *.war.zip file naming pattern
# The script will check whether Liberty is being used, and use the correct file naming pattern accordingly (.war or .war.zip).
warFileExtension=".war"

if war_location.find("clmServerTemplate/apps") != -1 or war_location.find("clmServerTemplate\\apps") != -1:
	warFileExtension=".war.zip"

#######################
# deploy all the apps #
#######################

for app in apps:

	try:
		print "Installing "+app
		full_path_to_app = war_location + "/" +app + warFileExtension

		# check to see if the .war for the expected app exists in the specified directory; if not, skip it.
		if (os.path.isfile(full_path_to_app)):

			if app != 'lqe' and app != 'ldx':
				attrs = ['-appname', app+".war", '-defaultbinding.virtual.host default_host -usedefaultbindings', '-node', node, '-server', server, '-contextroot',  "/"+app]
				error = AdminApp.install(full_path_to_app , attrs)
				AdminConfig.save()

			# Check to see if the CLM apps to be deployed contains any of the following:
			# JRS, LQE, LDX
			# These apps require additional configuration in order to be fully deployed
			# The additional configuration will be conducted below

			# Configuration for RS

			if app == "rs":
				AdminConfig.create('Library', AdminConfig.getid('/Cell:'+cell+'/Node:'+node+'/Server:'+server+'/'), '[[nativePath ""] [name "JRS Shared Library"] [isolatedClassLoader true] [description ""] [classPath "'+class_path+'"]]')
				AdminApp.edit('rs.war', '[ -MapSharedLibForMod [[ rs.war META-INF/application.xml "JRS Shared Library" ]]]' )
				dep = AdminConfig.getid('/Deployment:rs.war/')
				depObject = AdminConfig.showAttribute(dep, 'deployedObject')
				classldr = AdminConfig.showAttribute(depObject, 'classloader')
				AdminConfig.modify(classldr, [['mode', 'PARENT_LAST']])
				AdminConfig.save()

			# Configuration for LQE

			if app == 'lqe':
				AdminConfig.create('Property', '(cells/'+cell+'/nodes/'+node+'/servers/'+server+'|server.xml#JavaVirtualMachine_1183122130078)', '[[validationExpression ""] [name "lqe.config.location"] [description ""] [value "file:///'+configuration_path+'lqe"] [required "false"]]')
				AdminApp.install(war_location+'/lqe'+warFileExtension, '[ -usedefaultbindings -defaultbinding.virtual.host default_host -nopreCompileJSPs -distributeApp -nouseMetaDataFromBinary -nodeployejb -appname lqe.war -createMBeansForResources -noreloadEnabled -nodeployws -validateinstall warn -noprocessEmbeddedConfig -filepermission .*\.dll=755#.*\.so=755#.*\.a=755#.*\.sl=755 -noallowDispatchRemoteInclude -noallowServiceRemoteInclude -asyncRequestDispatchType DISABLED -nouseAutoLink -noenableClientModule -clientMode isolated -novalidateSchema -contextroot /lqe -MapModulesToServers [[ lqe lqe'+warFileExtension+',WEB-INF/web.xml WebSphere:cell='+cell+',node='+node+',server='+server+' ]] -CtxRootForWebMod [[ lqe lqe'+warFileExtension+',WEB-INF/web.xml /lqe ]]]' )
				AdminApp.edit('lqe.war', '[ -MapModulesToServers [[ lqe lqe'+warFileExtension+',WEB-INF/web.xml WebSphere:cell='+cell+',node='+node+',server='+server+' ]]]' )
				dep = AdminConfig.getid('/Deployment:lqe.war/')
				depObject = AdminConfig.showAttribute(dep, 'deployedObject')
				classldr = AdminConfig.showAttribute(depObject, 'classloader')
				policy = AdminConfig.showAttribute(depObject, 'warClassLoaderPolicy')
				AdminConfig.modify(classldr, [['mode', 'PARENT_LAST']])
				AdminConfig.modify(depObject, [['warClassLoaderPolicy', 'SINGLE']])
				lqeModule = AdminConfig.showAttribute(depObject, 'modules')
				lqeModule = lqeModule[1:len(lqeModule)-1]
				AdminConfig.modify(lqeModule, [['classloaderMode', 'PARENT_LAST']])

				AdminConfig.save()

			# Configuration for LDX

			if app == 'ldx':
				AdminConfig.create('Property', '(cells/'+cell+'/nodes/'+node+'/servers/'+server+'|server.xml#JavaVirtualMachine_1183122130078)', '[[validationExpression ""] [name "ldx.config.location"] [description ""] [value "file:///'+configuration_path+'ldx"] [required "false"]]')
				AdminApp.install(war_location+'/ldx'+warFileExtension, '[ -usedefaultbindings -defaultbinding.virtual.host default_host -nopreCompileJSPs -distributeApp -nouseMetaDataFromBinary -nodeployejb -appname ldx.war -createMBeansForResources -noreloadEnabled -nodeployws -validateinstall warn -noprocessEmbeddedConfig -filepermission .*\.dll=755#.*\.so=755#.*\.a=755#.*\.sl=755 -noallowDispatchRemoteInclude -noallowServiceRemoteInclude -asyncRequestDispatchType DISABLED -nouseAutoLink -noenableClientModule -clientMode isolated -novalidateSchema -contextroot /ldx -MapModulesToServers [[ ldx ldx'+warFileExtension+',WEB-INF/web.xml WebSphere:cell='+cell+',node='+node+',server='+server+' ]] -CtxRootForWebMod [[ ldx ldx'+warFileExtension+',WEB-INF/web.xml /ldx ]]]' )
				AdminApp.edit('ldx.war', '[ -MapModulesToServers [[ ldx ldx'+warFileExtension+',WEB-INF/web.xml WebSphere:cell='+cell+',node='+node+',server='+server+' ]]]' )
				dep = AdminConfig.getid('/Deployment:ldx.war/')
				depObject = AdminConfig.showAttribute(dep, 'deployedObject')
				classldr = AdminConfig.showAttribute(depObject, 'classloader')
				policy = AdminConfig.showAttribute(depObject, 'warClassLoaderPolicy')
				AdminConfig.modify(classldr, [['mode', 'PARENT_LAST']])
				AdminConfig.modify(depObject, [['warClassLoaderPolicy', 'SINGLE']])
				ldxModule = AdminConfig.showAttribute(depObject, 'modules')
				ldxModule = ldxModule[1:len(ldxModule)-1]
				AdminConfig.modify(ldxModule, [['classloaderMode', 'PARENT_LAST']])

				AdminConfig.save()


		else:
			print "  Info: Unable to locate the following application - skipping install:"
			print "  "+full_path_to_app
			print "  "
	except:
		_type_, _value_, _tbck_ = sys.exc_info()
		error = `_value_`
		print "Error Installing Application "
		print "Error Message = "+error
		returncode = 1
		#endIf
	#endTry
#endFor

print "The installed application(s) may be started from the Integrated Solutions Console in WebSphere."


# Example of how to set role/mapping for jts.war
# AdminApp.edit('jts.war', '[-MapRolesToUsers [["JazzAdmins" No No "" admins]["JazzDWAdmins" No No "" admins]["JazzUsers" No No "" users]["JazzGuests" No No "" users]["JazzProjectAdmins" No No "" admins]]]')
# AdminConfig.save()

# Notify the caller whether all applications were installed successfully
sys.exit(returncode)